@@ -1,48 +1,58 @@
 import { PropsWithChildren, useCallback, useEffect, useMemo, useState } from 'react';
+import { useFormContext } from 'react-hook-form';
 
 import { CoreApp } from '@grafana/data';
 import { config } from '@grafana/runtime';
 import { DataQuery } from '@grafana/schema';
-import { contextSrv } from 'app/core/services/context_srv';
 import { QueryLibraryContext, QueryLibraryDrawerOptions } from 'app/features/explore/QueryLibrary/QueryLibraryContext';
 
 import { SavedQuery, type OnSelectQueryType } from '../../features/explore/QueryLibrary/types';
 
+import { QueryDetails } from './QueryLibrary/QueryLibraryDetails';
 import { QueryLibraryInteractions } from './QueryLibraryAnalyticsEvents';
 import { QueryLibraryDrawer } from './QueryLibraryDrawer';
 import { QueryLibraryEditingHeader } from './QueryLibraryEditingHeader';
 import { SavedQueryButtons } from './SavedQueryButtons';
-import { QueryLibraryEventsPropertyMap } from './types';
+import { QueryLibraryEventsPropertyMap, QueryLibraryTab } from './types';
+import { getUserStorageFavorites, setUserStorageFavorites } from './utils/favorites';
+import { hasWritePermissions } from './utils/identity';
 
 import { getQueryLibraryDrawerAction, getQueryLibraryRenderContext } from './index';
 
 export function QueryLibraryContextProvider({ children }: PropsWithChildren) {
+  // SAVED QUERIES STATE
+  // Context / Drawer State
   const [isDrawerOpen, setIsDrawerOpen] = useState<boolean>(false);
+  const [context, setContext] = useState('unknown');
+  const [activeTab, setActiveTab] = useState(QueryLibraryTab.ALL);
   const [activeDatasources, setActiveDatasources] = useState<string[]>([]);
 
-  const [newQuery, setNewQuery] = useState<SavedQuery | undefined>(undefined);
-  // Use regular state for the callback
+  // Callback State
   const [onSelectQuery, setOnSelectQuery] = useState<OnSelectQueryType>(() => () => {});
-  const [context, setContext] = useState('unknown');
   const [onSave, setOnSave] = useState<(() => void) | undefined>(undefined);
+  const [userFavorites, setUserFavorites] = useState<{ [key: string]: boolean }>({});
   // Guard function that checks if drawer is allowed to close (returns true to allow, false to prevent)
   const [closeGuard, setCloseGuard] = useState<() => boolean>(() => () => true);
 
-  // Enhanced drawer options
+  // Query Specific State
+  const [newQuery, setNewQuery] = useState<SavedQuery | undefined>(undefined);
   const [highlightedQuery, setHighlightedQuery] = useState<string | undefined>(undefined);
+  const [isEditingQuery, setIsEditingQuery] = useState(!!newQuery);
+
+  const QueryDetailsFormMethods = useFormContext<QueryDetails>();
 
-  // Auto-clear highlight after 3 seconds
   useEffect(() => {
-    if (highlightedQuery) {
-      const timer = setTimeout(() => {
-        setHighlightedQuery(undefined);
-      }, 2000);
+    getUserStorageFavorites().then((value) => setUserFavorites(value));
+  }, []);
 
-      return () => clearTimeout(timer);
+  useEffect(() => {
+    // Update tab to All if editing a new query
+    if (newQuery && activeTab !== QueryLibraryTab.ALL) {
+      setActiveTab(QueryLibraryTab.ALL);
     }
-    return undefined;
-  }, [highlightedQuery]);
+  }, [newQuery, activeTab, setActiveTab]);
 
+  // SAVED QUERIES CALLBACKS
   const triggerAnalyticsEvent = useCallback(
     (
       handleAnalyticEvent: (properties?: QueryLibraryEventsPropertyMap) => void,
@@ -56,10 +66,55 @@ export function QueryLibraryContextProvider({ children }: PropsWithChildren) {
     [context]
   );
 
+  const onFavorite = useCallback(
+    async (uid: string) => {
+      const prevFavorites = { ...userFavorites };
+
+      const newUserFavorites = { ...prevFavorites, [uid]: true };
+      try {
+        setUserFavorites(newUserFavorites);
+        await setUserStorageFavorites(newUserFavorites);
+        triggerAnalyticsEvent(QueryLibraryInteractions.favoriteQueryClicked, { favoriteState: true });
+      } catch (e) {
+        setUserFavorites(prevFavorites);
+      }
+    },
+    [userFavorites, triggerAnalyticsEvent]
+  );
+
+  const onUnfavorite = useCallback(
+    async (uid: string) => {
+      const prevFavorites = { ...userFavorites };
+
+      const newUserFavorites = { ...prevFavorites };
+      delete newUserFavorites[uid];
+      try {
+        setUserFavorites(newUserFavorites);
+        await setUserStorageFavorites(newUserFavorites);
+        triggerAnalyticsEvent(QueryLibraryInteractions.favoriteQueryClicked, { favoriteState: false });
+      } catch (e) {
+        setUserFavorites(prevFavorites);
+      }
+    },
+    [userFavorites, triggerAnalyticsEvent]
+  );
+
+  // Auto-clear highlight after 3 seconds
+  useEffect(() => {
+    if (highlightedQuery) {
+      const timer = setTimeout(() => {
+        setHighlightedQuery(undefined);
+      }, 2000);
+
+      return () => clearTimeout(timer);
+    }
+    return undefined;
+  }, [highlightedQuery]);
+
   const openDrawer = useCallback(
     ({ datasourceFilters, onSelectQuery, options, query }: QueryLibraryDrawerOptions) => {
-      // this means the user is trying to add a query but is a viewer
-      if (!!query && contextSrv.hasRole('Viewer')) {
+      // this means the user is trying to save a query but doesn't have write permission
+      if (!!query && !hasWritePermissions()) {
         return;
       }
 
@@ -113,6 +168,7 @@ export function QueryLibraryContextProvider({ children }: PropsWithChildren) {
       // Reset the callback to no-op function
       setOnSelectQuery(() => () => {});
       setNewQuery(undefined);
+      setIsEditingQuery(false);
       setIsDrawerOpen(false);
       setCloseGuard(() => () => true);
 
@@ -120,6 +176,8 @@ export function QueryLibraryContextProvider({ children }: PropsWithChildren) {
       setHighlightedQuery(undefined);
 
       if (isEditingQuery) {
+        QueryDetailsFormMethods?.reset();
+
         triggerAnalyticsEvent(QueryLibraryInteractions.queryLibraryClosedToEditQueryInExplore);
       } else if (!isSelectingQuery && !newQuery) {
         triggerAnalyticsEvent(QueryLibraryInteractions.queryLibraryClosedWithoutSelection);
@@ -127,30 +185,36 @@ export function QueryLibraryContextProvider({ children }: PropsWithChildren) {
         triggerAnalyticsEvent(QueryLibraryInteractions.queryLibraryClosedWithoutSavingNewQUery);
       }
     },
-    [newQuery, triggerAnalyticsEvent, closeGuard]
+    [newQuery, QueryDetailsFormMethods, closeGuard, triggerAnalyticsEvent]
+  );
+
+  const onAddHistoryQueryToLibrary = useCallback(
+    (newQuery: SavedQuery) => {
+      setNewQuery(newQuery);
+      setIsEditingQuery(true);
+      triggerAnalyticsEvent(QueryLibraryInteractions.saveRecentQueryClicked);
+    },
+    [triggerAnalyticsEvent]
+  );
+
+  const onTabChange = useCallback(
+    (tab: QueryLibraryTab) => {
+      triggerAnalyticsEvent(QueryLibraryInteractions.tabClicked, { tab });
+      setActiveTab(tab);
+    },
+    [triggerAnalyticsEvent, setActiveTab]
   );
 
   const contextVal = useMemo(
     () => ({
-      isDrawerOpen,
-      openDrawer,
       closeDrawer,
-      triggerAnalyticsEvent,
-      renderSavedQueryButtons: (
-        query: DataQuery,
-        app?: CoreApp,
-        onUpdateSuccess?: () => void,
-        onSelectQuery?: (query: DataQuery) => void,
-        datasourceFilters?: string[]
-      ) => (
-        <SavedQueryButtons
-          query={query}
-          app={app}
-          onUpdateSuccess={onUpdateSuccess}
-          onSelectQuery={onSelectQuery}
-          datasourceFilters={datasourceFilters || []}
-        />
-      ),
+      onAddHistoryQueryToLibrary,
+      onFavorite,
+      onSave,
+      onSelectQuery,
+      onTabChange,
+      onUnfavorite,
+      openDrawer,
       renderQueryLibraryEditingHeader: (
         query: DataQuery,
         app?: CoreApp,
@@ -168,22 +232,59 @@ export function QueryLibraryContextProvider({ children }: PropsWithChildren) {
           onSelectQuery={onSelectQuery}
         />
       ),
-      queryLibraryEnabled: Boolean(config.featureToggles.queryLibrary),
-      context,
+      renderSavedQueryButtons: (
+        query: DataQuery,
+        app?: CoreApp,
+        onUpdateSuccess?: () => void,
+        onSelectQuery?: (query: DataQuery) => void,
+        datasourceFilters?: string[],
+        parentRef?: React.RefObject<HTMLElement>
+      ) => (
+        <SavedQueryButtons
+          query={query}
+          app={app}
+          onUpdateSuccess={onUpdateSuccess}
+          onSelectQuery={onSelectQuery}
+          datasourceFilters={datasourceFilters || []}
+          parentRef={parentRef}
+        />
+      ),
+      setActiveTab,
+      setIsEditingQuery,
       setNewQuery,
-      onSave,
-      onSelectQuery,
+      triggerAnalyticsEvent,
+      activeDatasources,
+      activeTab,
+      context,
+      highlightedQuery,
+      isDrawerOpen,
+      isEditingQuery,
+      newQuery,
+      queryLibraryEnabled: Boolean(config.featureToggles.queryLibrary),
+      userFavorites,
       setCloseGuard: handleSetCloseGuard,
     }),
     [
-      isDrawerOpen,
-      openDrawer,
       closeDrawer,
-      context,
-      setNewQuery,
+      onAddHistoryQueryToLibrary,
+      onFavorite,
       onSave,
-      triggerAnalyticsEvent,
       onSelectQuery,
+      onTabChange,
+      onUnfavorite,
+      openDrawer,
+      setActiveTab,
+      setIsEditingQuery,
+      setNewQuery,
+      triggerAnalyticsEvent,
+      activeDatasources,
+      activeTab,
+      context,
+      highlightedQuery,
+      isDrawerOpen,
+      isEditingQuery,
+      newQuery,
+      userFavorites,
       handleSetCloseGuard,
     ]
   );
@@ -191,13 +292,7 @@ export function QueryLibraryContextProvider({ children }: PropsWithChildren) {
   return (
     <QueryLibraryContext.Provider value={contextVal}>
       {children}
-      <QueryLibraryDrawer
-        isOpen={isDrawerOpen}
-        close={() => closeDrawer(false)}
-        activeDatasources={activeDatasources}
-        highlightedQuery={highlightedQuery}
-        newQuery={newQuery}
-      />
+      <QueryLibraryDrawer />
     </QueryLibraryContext.Provider>
   );
 }
